/*
NCSA Portfolio
Copyright 1997-1998 The Board of Trustees of the University of Illinois
All Rights Reserved 

NCSA Portfolio software, binary, Java-bytecode, and source (hereafter,
Software) is copyrighted by The Board of Trustees of the University of
Illinois (hereafter, UI), and ownership remains with the UI.

The UI grants you (hereafter, Licensee) a license to use the Software for
academic, research and internal business purposes only, without a fee.
Licensee may distribute the binary, Java byte-code, and source code (if
released) to third parties provided that the copyright notice and this
statement appears on all copies and that no charge is associated with such
copies.

Licensee may make derivative works.  However, if Licensee distributes any
derivative work based on or derived from the Software, then Licensee will
(1) notify NCSA regarding its distribution of the derivative work, and (2)
clearly notify users that such derivative work is a modified version and
not the original Software (NCSA Portfolio) distributed by the UI.

Any Licensee wishing to make commercial use of the Software should contact
the UI, c/o NCSA, to negotiate an appropriate license for such commercial
use.  Commercial use includes (1) integration of all or part of the
Software into a product for sale or license by or on behalf of Licensee to
third parties, or (2) distribution of the Software to third parties that
need it to utilize a commercial product sold or licensed by or on behalf of
Licensee. 

UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY,
INCLUDING, WITHOUT LIMITATION, NO WARRANTY AGAINST INFRIGEMENT OF ANY
INTELLECTUAL PROPERY RIGHTS.  THE UI SHALL NOT BE LIABLE FOR ANY DAMAGES
SUFFERED BY THE USERS OF THIS SOFTWARE.

By using or copying this Software, Licensee agrees to abide by the
copyright law and all other applicable laws of the U.S. including, but not
limited to, export control laws, and the terms of this license.  UI shall
have the right to terminate this license immediately by written notice upon
Licensee's breach of, or non-compliance with, any of its terms.  Licensee
may be held legally responsible for any copyright infringement that is
caused or encouraged by Licensee's failure to abide by the terms of this
license.

U.S. GOVERNMENT RESTRICTED RIGHTS 

The Software and/or user documentation are provided with RESTRICTED AND
LIMITED RIGHTS.  Use, duplication or disclosure by or to the United States
government shall be subject to restrictions as set forth in subparagraph
(c)(1)(ii) of Defense Federal Acquisition Regulations Supplement (DFARS)
Section 252.227-7013 for Department of Defense contracts and as set forth
in Federal Acquisition Regulations (FAR) Section 52.227-19 for civilian
agency contracts or any successor regulations.  Contractor/Manufacturer is
The Board of Trustees of the University of Illinois, Urbana, Illinois 61801.

NCSA is a registered trademark owned by The Board of Trustees of the
University of Illinois.
*/
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.net.*;
import ncsa.j3d.ui.record.*;


public class DisplayMenuBar extends MenuBar {

	public DisplayMenuBar(Display display) {
		super();
		Frame frame = display.getFrame();
		Menu file = new Menu("File");
		MenuItem mi = null;
		mi = (MenuItem) file.add(new MenuItem("Load"));
		mi.addActionListener( new LoadListener(display) );

		mi = (MenuItem) file.add(new MenuItem("Load URL"));
		mi.addActionListener( new LoadURLListener(display) );

		mi = (MenuItem) file.add(new MenuItem("Toggle Stereo"));
		mi.addActionListener( new StereoListener(display) );

		mi = (MenuItem) file.add(new MenuItem("About"));
		mi.addActionListener( new AboutListener(frame) );

		mi = (MenuItem) file.add(new MenuItem("Quit"));
		mi.addActionListener( new QuitListener() );

		add(file);
	}
}

class QuitListener implements ActionListener {

	public QuitListener() {
	}

	public void actionPerformed(ActionEvent e) {
		System.exit(0);
	}
}

class AboutListener implements ActionListener {
	Dialog aboutBox = null;
	Frame frame = null;

	public AboutListener(Frame f) {
		frame = f;
	}

	public void actionPerformed(ActionEvent e) {
		if(aboutBox != null) {
			aboutBox.pack();
			aboutBox.show();
			return;
		}
		aboutBox = new Dialog(frame, "About NCSA Display", false);
		Panel groupPanel = new Panel(new GridLayout(8,1));
		groupPanel.add(new Label("NCSA Display 1.0 beta"));
		groupPanel.add(new Label("National Center for Supercomputing Applications"), BorderLayout.CENTER);
		groupPanel.add(new Label("University of Illinois at Urbana-Champaign"), BorderLayout.CENTER);
		groupPanel.add(new Label("NCSA Java 3D Group"));
		groupPanel.add(new Label("NCSA Display created by"));
		groupPanel.add(new Label("Stephen R. Pietrowicz - srp@ncsa.uiuc.edu"));
		groupPanel.add(new Label("Chris Heistad - heistad@ncsa.uiuc.edu"));
 		Panel buttonPanel = new Panel();

		Button button = (Button) buttonPanel.add(new Button("I didn't know that"));
		button.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            aboutBox.setVisible(false);
                        }
                    });
		groupPanel.add(buttonPanel);

		aboutBox.add(groupPanel);

                aboutBox.pack();
                aboutBox.show();
	}
}


class LoadListener implements ActionListener {
	FileDialog loadBox;
	Frame frame = null;
	Display display;

	public LoadListener(Display d) {
		display = d;
		frame = display.getFrame();
	}

	public void actionPerformed(ActionEvent e) {
		loadBox = new FileDialog(frame, "Load file", FileDialog.LOAD);
                loadBox.pack();
                loadBox.show();
		String dir = loadBox.getDirectory();
		String file = loadBox.getFile();
		if ((dir == null) || (file == null))
			return;
		String filename =dir+file;
		display.load(filename);
	}
}



class StereoListener implements ActionListener {
        FileDialog saveBox = null;
        Frame frame = null;
	RecordableCanvas3D canvas = null;

        public StereoListener(Display d) {
		canvas = d.getRecordableCanvas3D();
        }

        public void actionPerformed(ActionEvent e) {
                boolean stereoOn = canvas.getStereoEnable();
                if (stereoOn)
                        canvas.setStereoEnable(false);
                else
                        canvas.setStereoEnable(true);
        }
}

class LoadURLListener implements ActionListener {
	Dialog loadBox;
        TextField tf;
	Frame frame = null;
	Display display;

	public LoadURLListener(Display d) {
		display = d;
		frame = display.getFrame();
	}

	public void actionPerformed(ActionEvent e) {

		loadBox = new Dialog(frame, "Load URL");
		Panel boxPanel = new Panel();
		Panel buttonPanel = new Panel();

		boxPanel.add(new Label("URL: "));
		tf = new TextField(60);
		tf.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
			    URL URLname;
			    loadBox.setVisible(false);
			    try {
			      URLname = new URL(tf.getText());
			    } catch (MalformedURLException URLex) { 
			      System.out.println("Malformed URL: " +
						 tf.getText());
			      return;
			    }
			    display.load(URLname);
                        }
                    });
	        boxPanel.add(tf);

		Button okayButton = new Button("Okay");
		okayButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
			    URL URLname;
			    loadBox.setVisible(false);
			    try {
			      URLname = new URL(tf.getText());
			    } catch (MalformedURLException URLex) { 
			      System.out.println("Malformed URL: " +
						 tf.getText());
			      return;
			    }
			    display.load(URLname);
                        }
                    });
		buttonPanel.add(okayButton);

		Button cancelButton = new Button("Cancel");
		cancelButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e) {
                            loadBox.setVisible(false);
                        }
                    });
		buttonPanel.add(cancelButton);

		Panel mainPanel = new Panel(new GridLayout(2,1));
		mainPanel.add(boxPanel);
		mainPanel.add(buttonPanel);
		loadBox.add(mainPanel);
                loadBox.pack();
                loadBox.show();

	}
}


