import java.awt.*;
import java.awt.event.WindowListener;
import java.awt.event.WindowEvent;
import javax.comm.*;
import java.util.Enumeration;
import ncsa.devices.wand.*;
import ncsa.devices.bird.*;
import ncsa.j3d.ui.PortfolioProperties;

public class WandVision extends Panel implements WindowListener {
  private WandDevice wd;
  private BirdAngleDevice bad;
  private boolean going;
  private Frame f;
  private Panel flock[];
  private TextField fText[][];

  private boolean wand;
  private boolean bird;

  private int flocks;

  public WandVision() {
    wd = new WandDevice();
    bad = new BirdAngleDevice();
    going = true;
    init();
    flocks = bad.getSensorCount();
    flock = new Panel[flocks];
    fText = new TextField[flocks][6];
    for (int y = 0; y < flocks; y++) {
      flock[y] = new Panel();
      for (int x = 0; x < 6; x++) {
        fText[y][x] = new TextField("0.0",20);
        fText[y][x].setForeground(new Color(1.0f, 1.0f, 0.0f));
        fText[y][x].setEditable(false);
        fText[y][x].setVisible(true);
      }
    }
    setupFrame();
  }

  public void windowActivated(WindowEvent e) { }
  public void windowClosed(WindowEvent e) { }
  public void windowOpened(WindowEvent e) { going = true; repaint(); }
  public void windowDeiconified(WindowEvent e) { going = true; repaint(); }
  public void windowIconified(WindowEvent e) { going = false; }
  public void windowDeactivated(WindowEvent e) { }
  public void windowClosing(WindowEvent e) { 
    going = false;
    wd = null;
    f.dispose();
    System.exit(10);
  }

  public int button(int x) { return wd.button(x); }

  public void paint(Graphics g) {
    pollAndProcessInput();

    if (bird) { 
      for (int x = 0; x < flocks; x++) {
        fText[x][0].setText("X:  " + Float.toString(bad.getX(x)));
        fText[x][1].setText("Y:  " + Float.toString(bad.getY(x)));
        fText[x][2].setText("Z:  " + Float.toString(bad.getZ(x)));
        fText[x][3].setText("XR: " + Float.toString(bad.getXR(x)));
        fText[x][4].setText("YR: " + Float.toString(bad.getYR(x)));
        fText[x][5].setText("ZR: " + Float.toString(bad.getZR(x)));
      }
    }

    if (wand) {
      // Buttons
      for (int x = 0; x < 3; x++) {
        if (button(x) == 1) {
          g.setColor(new Color(1.0f, 0.0f, 0.0f));
        } else {
          float shade = ((x + 1) % 3)/2.0f;
          g.setColor(new Color(shade, shade, shade));
        }
        if (x == 0)      { g.fillOval(10,40,25,25); }
        else if (x == 1) { g.fillOval(50,25,25,25); }
        else if (x == 2) { g.fillOval(90,40,25,25); }
      }
  
      // Joystick
      g.setColor(new Color(0.0f,0.0f,0.0f));
      g.drawOval(37,70,50,50);
      g.setColor(new Color(1.0f, 1.0f, 0.0f));
  
      double x = wd.valuator(0);
      double y = wd.valuator(1);
  
      double theta = Math.atan2(y,x);
  
      g.drawLine(62,95,(int) (62 + 25 *  Math.abs(x) * Math.cos(theta) ), 
                       (int) (95 + 25 * -Math.abs(y) * Math.sin(theta) ) );
  
      g.setColor(new Color(0.0f,0.0f,0.0f));
    }

    if (going) {
      repaint(100);
    }
  }

  public void pollAndProcessInput() {
    if (wand) { wd.pollAndProcessInput(); }
    if (bird) { bad.pollAndProcessInput(); }
  }

  private void init() {
    wand = true;
    bird = true;

    try {
      wd.initializePortfolioDevice("WandDriver");
    } catch (NullPointerException npe) {
      wand = false;
      System.err.println("No wand found.");
    }

    try {
      bad.initializePortfolioDevice("BirdAngleDevice");
    } catch (NullPointerException npe) {
      bird = false;
      System.err.println("No bird found.");
    }
  }

  private void setupFrame() {

    f = new Frame("WandVision");
    if (wand && !bird) {
      f.setSize(120,160);
    } else if (wand && bird) {
      f.setSize((200 * flocks) + 120,200);
    } else {
      f.setSize((200 * flocks),200);
    }
    f.setResizable(false);
    f.setBackground(new Color(0.0f,0.4f,0.8f));
    f.addWindowListener(this);
    f.setLayout(new GridLayout(1,2));

    if (wand) {
      setVisible(true);
      this.setSize(140,160);
      f.add(this);
    }

    if (bird) {
      for (int y = 0; y < flocks; y++) {
        flock[y].setBackground(new Color(0.0f,0.4f,0.8f));
        for (int x = 0; x < 6; x++) {
          flock[y].add(fText[y][x]);
        }
        flock[y].setVisible(true);
        f.add(flock[y]);
      }
    }

    f.setVisible(true);
  }

  static public void main(String[] args) {
    WandVision w = new WandVision();
  }
}
